#!/bin/sh
#
#    Copyright (c) 1996-2004 by Brocade Communications Systems, Inc.
#    All rights reserved.
#
#    faninit.sh

#
# MFG SEEPROM Initialization script (FAN)
#
# 02/20/04    initial version
#

######################################################################
# constants
maxPN=14 # max PN char can be entered (13 + <cr>)
maxSN=12 # max SN char can be entered (11 + <cr>)

default_PN="60-0003377-01"   # FAN PN
default_SN="13460911"        # FAN SN
######################################################################

######################################################################
#  utility functions
######################################################################

#
# userInputPL()
#
# $1 : display name for input
# $2 : default value
userInputPL()
{
    ##################################################################
    # do nothing if the variable is set
    if [ $boardPL != 0 ]
    then
	return
    fi
    ##################################################################

    ##################################################################
    # set the platform with current SWBD
    boardPL=`getSWBD`
    ##################################################################
}

#
# userInputPN()
#
# $1 : display name for input
# $2 : default value
userInputPN()
{
    ##################################################################
    # do nothing if the variable is set
    if [ $boardPN != 0 ]
    then
	return
    fi
    ##################################################################

    ##################################################################
    # get input from user
    echo "Enter $1 (max 13 char)[$2]:"
    read line
    ##################################################################

    if [ $line ]
    then
	##############################################################
	# check input format
	nc=`echo $line | wc -c`
	if [ $nc -gt $maxPN ]
	then
	    echo "$2 input exceeded 13 characters."
	    exit 1
	else
	    boardPN=$line
	fi
	##############################################################
    else
	echo "No $1 entered, default to $2."
	boardPN=$2
    fi
}


#
# userInputSN()
#
# $1 : display name for input
# $2 : default value
userInputSN()
{
    ##################################################################
    # do nothing if the variable is set
    if [ $boardSN != 0 ]
    then
	return
    fi
    ##################################################################

    ##################################################################
    # get input from user
    echo "Enter $1 (max 11 char)[$2]:"
    read line
    ##################################################################

    if [ $line ]
    then
	##############################################################
	# check input format
	nc=`echo $line | wc -c`
	if [ $nc -gt $maxSN -o $nc -lt 6 ]
	then
	    echo "$2 input invalid length (6 - 11 characters required)."
	    exit
	else
	    boardSN=$line
	fi
	##############################################################
    else
	echo "No $1 entered, default to $2."
	boardSN=$2
    fi
}


#
# process_command()
#
# $* : command line
process_command()
{
    local message
    message=$1
    shift

    $*
    stat=$?

    if [ $stat -ne 0 ]
    then
	echo "FRUINIT ERROR : $*"
    fi

    status=`expr $status + $stat`
}


######################################################################
#  main
######################################################################

######################################################################
# source diagcommon
home="/fabos/share"
util="diagcommon.sh"
ok=0
for f in "./$util" "$FABOSHOME/share/$util" "$home/$util" ; do
    if [ -r $f ] ; then
	. $f
	ok=1
	break;
    fi
done
if [ $ok -ne 1 ] ; then
    err "Could not locate $util"
    exit 3
fi
######################################################################

#
# variables
#
boardPL=0  # platform
boardPN=$default_PN  # board PN
boardSN=$default_SN  # board SN
mfgDate=0  # manufacture date
mfgMonth=0 # manufacture month
mfgYear=0  # manufacture year

status=0

# find fruinit command
cmd=`commandPath fruinit`

#
#  command line parsing
#
while [ $# != 0 ]
do
    case $1 in 

    ver)  
	echo "faninit.sh : version 0.5"
	exit 0
	;;

    help)
	echo "faninit.sh : script to init FAN"
	echo " "
	echo "Usage: faninit.sh [help] [ver] [-p <swbd>] [-pn <n>] [-sn <n>] [-dd <n>] [-mm <n>][-yy <n>]"
	echo "	where:"
	echo "	  help          Display Usage"
	echo "	  ver           Version"
	echo "	  -p		Platform"
	echo "	  -pn <n>       FAN part number"
	echo "	  -sn <n>       FAN serial number"
	echo "	  -dd <n>	Manufacture Date [system]"
	echo "	  -mm <n>	Manufacture Month [system]"
	echo "	  -yy <n>	Manufacture Year [system]"
	echo " "
	exit 0
	;;

    -p)  # platform
	shift
	if [ $1 ]
	then
	    boardPL=$1
	else
	    ArgErr="-p"
	fi
	;;
	
    -pn) # FAN part number
	shift
	if [ $1 ]
	then
	    boardPN=$1
	else
	    ArgErr="-pn"
	fi
	;;

    -sn) # FAN serial number
	shift
	if [ $1 ]
	then
	    boardSN=$1

	    ##########################################################
	    # input validity check(11 char max)
	    nc=`echo $boardSN | wc -c`
	    if [ $nc -gt $maxSN -o $nc -lt 6 ]
	    then
		boardSN=0
	    fi
	    ##########################################################
	else
	    ArgErr="-sn"
	fi
	;;

    -dd) # MFG date
	shift
	if [ $1 ]
	then
	    mfgDate=$1
	else
	    ArgErr="-dd"
	fi
	;;

    -mm) # MFG month
	shift
	if [ $1 ]
	then
	    mfgMonth=$1
	else
	    ArgErr="-mm"
	fi
	;;

    -yy) # MFG year
	shift
	if [ $1 ]
	then
	    mfgYear=$1
	else
	    ArgErr="-yy"
	fi
	;;

    *)
	echo "ERROR:  ABORTED - Invalid token <$1> entered"
	exit 1
	;;

    esac

    ##################################################################
    # check for error condition
    if test $ArgErr
    then
	echo "ERROR: ABORTED - Token <$ArgErr> requires an arg token"
	exit 1
    fi
    ##################################################################

    shift
done


#
#  commence script operation
#

######################################################################
# calculate date information if not available
if [ $mfgMonth = 0 ]
then
    mfgMonth=`/bin/date '+ %m' | sed "s/ 0/ /g" | sed "s/ //g"`
fi

if [ $mfgDate = 0 ]
then
    mfgDate=`/bin/date '+ %d' | sed "s/ 0/ /g" | sed "s/ //g"`
fi

if [ $mfgYear = 0 ]
then
    tempYear=`/bin/date '+ %y'`
    mfgYear=`expr $tempYear + 2000`
fi
######################################################################

######################################################################
#  set platform
userInputPL "Platform"  # set value (no user input)
######################################################################

######################################################################
#  get FAN PN & SN
userInputPN "FAN PN" $default_PN
userInputSN "FAN SN" $default_SN
######################################################################


#
#  execute fruinit with parameters
#
process_command "" $cmd -p $boardPL -c INIT -t FAN -u 1
process_command "" $cmd -p $boardPL -c INIT -t FAN -u 2
process_command "" $cmd -p $boardPL -c INIT -t FAN -u 3

process_command "" $cmd -p $boardPL -c INITSIG -t FAN -u 1

process_command "" $cmd -p $boardPL -c SET -t FAN -u 1 -k PARTNO -v $boardPN
process_command "" $cmd -p $boardPL -c SET -t FAN -u 1 -k SERIAL -v $boardSN
process_command "" $cmd -p $boardPL -c SET -t FAN -u 1 -k MFGDAY -v $mfgDate
process_command "" $cmd -p $boardPL -c SET -t FAN -u 1 -k MFGMONTH -v $mfgMonth
process_command "" $cmd -p $boardPL -c SET -t FAN -u 1 -k MFGYEAR -v $mfgYear

process_command "" $cmd -p $boardPL -c INITSIG -t FAN -u 2

process_command "" $cmd -p $boardPL -c SET -t FAN -u 2 -k PARTNO -v $boardPN
process_command "" $cmd -p $boardPL -c SET -t FAN -u 2 -k SERIAL -v `expr $boardSN + 1`
process_command "" $cmd -p $boardPL -c SET -t FAN -u 2 -k MFGDAY -v $mfgDate
process_command "" $cmd -p $boardPL -c SET -t FAN -u 2 -k MFGMONTH -v $mfgMonth
process_command "" $cmd -p $boardPL -c SET -t FAN -u 2 -k MFGYEAR -v $mfgYear

process_command "" $cmd -p $boardPL -c INITSIG -t FAN -u 3

process_command "" $cmd -p $boardPL -c SET -t FAN -u 3 -k PARTNO -v $boardPN
process_command "" $cmd -p $boardPL -c SET -t FAN -u 3 -k SERIAL -v `expr $boardSN + 2`
process_command "" $cmd -p $boardPL -c SET -t FAN -u 3 -k MFGDAY -v $mfgDate
process_command "" $cmd -p $boardPL -c SET -t FAN -u 3 -k MFGMONTH -v $mfgMonth
process_command "" $cmd -p $boardPL -c SET -t FAN -u 3 -k MFGYEAR -v $mfgYear

######################################################################
# translate the result
if [ $stat = 0 ]
then
    status=PASSED
else
    status=FAILED
fi
######################################################################

######################################################################
# display and exit
echo "faninit.sh: DONE - $status"
exit $stat
######################################################################


